--[[
    File: tuf_party_auras.lua
    Author: Tuck
    Description: This file contains all funtions that handle broadcasting of aura information within a party.
]]
 
local tuf_network = TuckUF.network
local tuf_control = TuckUF.controlframe
local CommChannel = CommChannel("1.0")

-- AuraTimes[target][aura] = {time, totaltime}
tuf_network.AuraTimes = {}
tuf_network.OwnAuraTimes = {}

-- ComboPoints[name] = number of CP
tuf_network.ComboPoints = {}

local TRACE = function(msg) ChatFrame1:AddMessage(msg) end

local rpc_iface = { }
local comm_channel = CommChannel:Create("PARTY", "TUF", rpc_iface) -- does not work with "RAID" when in a party but not in a raid

--[[
    *---------------------------------------------------------------------------------------------
    * Local functions
    *---------------------------------------------------------------------------------------------
]]

--[[
    Returns a (unique) network ID for the given unit ID
]]
local function unitid_to_networkid(unit_id)
    -- some info about the unit
	-- todo: locale independent info
	local unitType = UnitCreatureFamily(unit_id) or UnitCreatureType(unit_id) or UnitRace(unit_id) or ""
	local unitSex = UnitSex(unit_id) or ""
	local unitName = UnitName(unit_id) or ""
	local unitLevel = UnitLevel(unit_id) or 0
	
	-- for players, take their name, for other mobs, append some disambiguation information (never meant to work, anyway)
	if (UnitIsPlayer(unit_id)) then
		return "PC_"..unitName
	else
		return "NPC_"..unitName.."_"..unitType.."_"..unitSex.."_"..unitLevel
	end
end

--[[
    Returns the name of a unit given by its network ID
]]
local function networkid_to_name(network_id)
    return ""
end

--[[
    Does something meaningful
]]
local function update_aura_timetable(time_table, dt)
	-- update all targets
	for target_name, aura_tbl in pairs(time_table) do

		-- update all auras for that target
		local aura_count = 0
		for aura_name, aura_times in pairs(aura_tbl) do
		
            local permanent = (aura_times.time == 0 and aura_times.totaltime == 0)
            
			-- decrease time left, if not permanent
            if not permanent then
                aura_times.time = aura_times.time - dt
                
                -- see if there is some time left
                if aura_times.time < 0 then
                    aura_tbl[aura_name] = nil
                else
                    aura_count = aura_count + 1
                end
            else
                aura_count = aura_count + 1
            end
		end
		
		-- see if there are any auras for this target
		if aura_count == 0 then
			time_table[target_name] = nil
		end
	end
end

--[[
    Broadcasts aura info, if nessesary. Can also do nothing.
]]
local function send_aura(network_id, aura, time, totaltime)

    local permanent = (time == 0 and totaltime == 0)
	local send = false
	
	-- check for target entry
	if not tuf_network.OwnAuraTimes[network_id] then
		tuf_network.OwnAuraTimes[network_id] = {}
		send = true
	end
	
	-- check for aura entry
	if not tuf_network.OwnAuraTimes[network_id][aura] then
        -- if entry not present at target, send
		tuf_network.OwnAuraTimes[network_id][aura] = {["time"] = time, ["totaltime"] = totaltime}
		send = true
	end
    
    local own_aura_data = tuf_network.OwnAuraTimes[network_id][aura]

	-- check if times are up to date
	local dtime = abs(own_aura_data.time - time)
	local dtotaltime = abs(own_aura_data.totaltime - totaltime)
	if dtime > 0.5 or dtotaltime > 0.5 then
        -- permanent buffs are always up to date
        if not permanent then
            send = true
        end
	end
	
	-- update to latest time information
	own_aura_data.time = time
	own_aura_data.totaltime = totaltime
	
    -- if not up to date, send
	if send then
        comm_channel:Call("A", network_id, aura, time, totaltime)
		--TRACE("sent A: "..network_id.." ; "..aura.." ; "..time.." ; "..totaltime)
	end
end

--[[
    Broadcasts CP info
]]
local function send_combo_points(unit_name, number)
    comm_channel:Call("CP", unit_name, number)
    -- TRACE("sent CP: "..unit_name.." ; "..number)
end

--[[
    *---------------------------------------------------------------------------------------------
    * Remote Procedure Calls
    * ---------------------------------------------------------------------------------------------
]]

--[[
    RPC: Unit Aura
]]
function rpc_iface:A(network_id, aura, time, totaltime)
      
	-- create new entry for this target
	if not tuf_network.AuraTimes[network_id] then
		tuf_network.AuraTimes[network_id] = {}
	end
	
	-- fill entry (always overwrite)
	tuf_network.AuraTimes[network_id][aura] = {["time"] = time, ["totaltime"] = totaltime}

    -- update frames
    tuf_control:OnNetworkAura(networkid_to_name(network_id))
    
    -- TRACE("rpc_iface:A("..network_id..", "..aura..", "..time..", "..totaltime..")")
end

--[[
    RPC: Combo Points
]]
function rpc_iface:CP(unit_name, number)
    tuf_network.ComboPoints[unit_name] = number
    
    -- update frames
    tuf_control:OnNetworkComboPoint(unit_name)
    
    -- TRACE("rpc_iface:CP("..unit_name..", "..number..")")
end

--[[
    *---------------------------------------------------------------------------------------------
    * Public functions
    * ---------------------------------------------------------------------------------------------
]]


--[[
    Call this after you have received UNIT_AURA
]]
function TuckUF.network:OnUnitAura(unit_id)

    local network_id = unitid_to_networkid(unit_id)
        
    for i = 1, 40 do	
        -- buff
		local name, _, _, _, duration, timeleft = UnitBuff(unit_id, i)
        if name and duration then
            send_aura(network_id, name, timeleft, duration)
        end
        
        -- debuff
        local name, _, _, _, _, duration, timeleft = UnitDebuff(unit_id, i)
        if name and duration then
            send_aura(network_id, name, timeleft, duration)
        end
    end
end

--[[
    Call this after you have received PLAYER_COMBO_POINTS
]]
function TuckUF.network:OnPlayerComboPoints()
    local unit_name = UnitName("player")
    
    send_combo_points(unit_name, GetComboPoints())
end

--[[
    OnUpdate function. This function updates all stored information by advancing the time
]]
function TuckUF.network:OnUpdate(dt)
	update_aura_timetable(tuf_network.AuraTimes, dt)
	update_aura_timetable(tuf_network.OwnAuraTimes, dt)
end


--[[
    Returns timing information for the given aura
    unit_id is the blizzard unit ID
    returns nil if no information available
]]
function TuckUF.network:GetAuraTimeByUnitID(unit_id, aura_name)   
    local network_id = unitid_to_networkid(unit_id)
    
    local target_data = tuf_network.AuraTimes[network_id]
	if (target_data and target_data[aura_name]) then
		return target_data[aura_name].time, target_data[aura_name].totaltime
	else
		return nil
	end
end

--[[
    ...
]]
function TuckUF.network:GetComboPointsByUnitID(unit_id)   
    local unit_name = UnitName(unit_id)
    
    return tuf_network.ComboPoints[unit_name]
end

